
// Tipi numerici (oltre ad int)

public class Esempio1Sol {
	
	static int i;    // variabile intera	
	static long j;   // variabile intera a maggior precisione 
					 // (può rappresentare valori con più cifre)
	static float x;  // variabile in virgola mobile
	static double y; // variabile in virgola mobile a maggior precisione
					// (può rappresentare valori con più cifre significative)
	
	static double a;
	static double b;
	static int n;
	static int dentro;
	
	public static void main(String[] args) {
		
		// Un numero grande:
		i = 1234567890;
		// Il suo quadrato è troppo grande per un int, che ammette
		// un massimo di 2^31-1. Quindi, viene stampato un valore errato.
		System.out.println("i^2 = " + (i*i));
		
		// Usando un long, riusciamo a stampare il quadrato esatto.
		j = i;
		System.out.println("j^2 = " + (j*j));
		// Anche i long hanno un valore massimo (2^63-1), da non superare.

		// ------------------------------------------------
		
		// Le variabili in virgola mobile (di tipo float o double)
		// possono contenere numeri con cifre decimali dopo la virgola.
		x = 12.32f;  // La f indica una costante di tipo "float"
		
		// Gli operatori aritmetici (+,-,...) si possono usare anche
		// tra espressioni di tipo a virgola mobile.
		x = x + 2.5f;
		
		// Tra espressioni di tipo in virgola mobile, l'operatore /
		// effettua ovviamente la divisione e non il quoziente come tra interi.
		x = x / 3.73f;
		
		// Le costanti di tipo double non hanno la f in fondo.  
		y = 1.0;     // Senza f è un double
		
		// Le variabili in virgola mobile non rappresentano
		// perfettamente i numeri reali.
		y = 1.0;		
		y = y / 49.0;
		System.out.println("y = " + y); // y = 0.02040816326530612
		y = y * 49.0;
		System.out.println("y = " + y); // y = 0.9999999999999999
		// Il risultato non è 1.0 perché le operazioni come 1.0/49.0
		// non sono esatte, ma calcolano solo una parte delle cifre.
		
		// Di conseguenza, è meglio non controllare mai i numeri in virgola
		// mobile con confronti "esatti" come y == 1 oppure y != 1.
		// Visto che ci può sempre essere un qualche errore di
		// arrotondamento, questi controlli possono dare un risultato
		// inaspettato!
		if (y == 1.0) {
			System.out.println("Questo non viene stampato!");
		}
		// Piuttosto, controllate che la differenza tra y e 1 sia piccola
		// in valore assoluto.
		if (Math.abs(y - 1.0) < 0.00001) {
			System.out.println("Questo invece viene stampato!");
		}
		
		// -----------------------------------------------------
		
		// Java converte automaticamente un valore di un tipo a minor
		// al suo analogo in un tipo a maggior precisione.
		// Questo aspetto viene chiamato "promozione".
		j = i; // int promosso a long
		y = x; // float promosso a double
		
		// Java non effettua invece la conversione opposta, visto che
		// nel caso generale è impossibile.
		// i = j;   // errore di tipo
		
		// Se siamo sicuri che il valore del tipo più preciso è
		// rappresentabile anche in quello meno preciso, possiamo
		// forzare la conversione con un "cast":
		j = 123;
		i = (int) j;  // cast da long a int
		
		// Il cast a volte viene usato per richiedere la divisione e
		// non il quoziente.
		i = 1;      // intero 1
		y = i/2;    // Questo calcola il quoziente 1/2=0 e *poi* lo promuove!
		System.out.println("y = " + y);    // Stampa 0.0
		
		// Col cast si può forzare la promozione di i (e quindi di 2)
		// a double prima della divisione.
		y = ((double) i) / 2;
		System.out.println("y = " + y);    // Stampa 0.5

		// Alternativamente, usando una costante double 2.0 invece di quella
		// intera 2, si forza comunque la promozione di i.
		y = i / 2.0;
		System.out.println("y = " + y);    // Stampa 0.5
		
		// -----------------------------------------------------
		
		// La funzione Math.random() genera un double casuale tra 0 e 1,
		// in modo "uniforme".
		for (i=0 ; i<10 ; i++) {
			y = Math.random();
			System.out.println("Numero casuale: " + y);
		}
		
		// Esercizio 1.
		// Dato n>=0, generate n punti casuali (a,b) dentro un quadrato di lato
		// unitario. Contate quanti di questi hanno entrambe le coordinate <= 0.5,
		// cioè cadono nel quarto del quadrato che tocca l'origine.
		// Intuitivamente, in media, quanti punti siffatti dovremmo contare?
		// Stampate il numero di punti siffatti rapportati al numero totale
		// per diversi valori di n, e osservate a quanto converge il rapporto.
		//
		// Questo metodo per stimare l'area di un insieme generando punti casuali
		// viene chiamato metodo Monte Carlo.
		
		// Soluzione
		
		System.out.println("--------------------------------------------------");
		n = 10000;
		dentro = 0;
		for (i=0 ; i<n ; i++) {
			a = Math.random();
			b = Math.random();
			if (a <= 0.5 && b <= 0.5) {
				dentro++;
			}
		}
		System.out.println("Punti totali " + n + ", di cui dentro " + dentro);
		System.out.println("Stima dell'area del quarto di quadrato: " + (double) dentro/n);
		
		
		// Esercizio 2.
		// Ripetete l'esercizio di prima contando invece i punti che distano
		// dal centro del quadrato al più 0.5 unità.
		// Intuitivamente, in media, quanti punti siffatti dovremmo contare?
		//
		// Se vi serve, potete usare Math.sqrt() per calcolare la radice
		// quadrata, ma è possibile fare a meno di usare tale funzione.

		// Soluzione
		
		System.out.println("--------------------------------------------------");
		n = 10000000;
		dentro = 0;
		for (i=0 ; i<n ; i++) {
			a = Math.random();
			b = Math.random();
			// Equivalente a Math.sqrt((a-0.5)*(a-0.5) + (b-0.5)*(b-0.5)) <= 0.5
			if ((a-0.5)*(a-0.5) + (b-0.5)*(b-0.5) <= 0.25) {
				dentro++;
			}
		}
		System.out.println("Punti totali " + n + ", di cui dentro " + dentro);
		System.out.println("Stima dell'area del cerchio inscritto: " + (double) dentro/n);
		System.out.println("Area * 4 : " + 4.0 * (double) dentro/n);
		
		// Esercizio 3.
		// Provate ad usare lo stesso metodo per stimare l'integrale della 
		// funzione seno (Math.sin) tra 0 e pi greco (Math.PI).
		// Sfruttate il fatto che il grafico del seno è incluso nel rettangolo
		// di base [0, pi greco] e altezza [0,1]. Generate n punti nel
		// rettangolo e contate quanti cadono sotto la curva.

		// Soluzione
		
		System.out.println("--------------------------------------------------");
		n = 10000000;
		dentro = 0;
		for (i=0 ; i<n ; i++) {
			a = Math.random() * Math.PI;
			b = Math.random();
			if (b <= Math.sin(a)) {
				dentro++;
			}
		}
		System.out.println("Punti totali " + n + ", di cui dentro " + dentro);
		System.out.println("Stima dell'integrale: " + ((double) dentro/n) * Math.PI);	
	}
}
